{
============================
SQL Query Parameters Manager
============================

Copyright 12.1997 By MAD Soft

Telephone: Bulgaria, Sofia, 37-06-23
E_Mail:    NMMM@NSI.BG
           NMMM@HotMail.Com
           NMMM@WebMail.Co.ZA
Web:       Http:\\WWW.NSI.BG\NMMM\Home.Htm

Versions:
   10.1997 1.0 Idea
   01.1998 1.1 Create
   01.1998 1.2 ReLooking
   03.1998 2.0 Multi Objects

Object tree:

   AbstractParamManager(8)=-----WinParamManager=---MadWinParamManager
                           \
                            ----ListParamManager
 }

{
Param Format:
-------------

  $SPARAMNAME
  ||\_______/
  ||    |
  ||    +-Name of parm
  ||
  |+------Type of param
  +-------Special symbol "$"

Param Types:
------------
S   String
I   Integer
W   Word
F   Float
D   Date
T   Time
B   Boolean
}

unit Qparam;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, DB, DBTables, Grids,
  QParamF,    {There is SQL param form}
  KeyArr;

Const ParamManagerSymbol = '$';

type
  TAbstractParamManager = class(TComponent)
  private
    { Private declarations }
  protected
    { Protected declarations }

    Accepted   : Boolean;
    M          : TKeyArr;

  public
    { Public declarations }
    Constructor Create(AOwner:TComponent); OverRide;
    Destructor  Destroy; OverRide;

    Procedure   Clear;               {Clear Parameters}
    Procedure   Get(Q:TQuery);       {Get Parameters From Query}

    Procedure   Action; Virtual;     {Go Action}

    Procedure   Accept(Q:TQuery);    {Accept Parameters T Query}

  published
    { Published declarations }
    Property    IsAccepted : Boolean read Accepted;
  end;

  TWinParamManager = class(TAbstractParamManager)
  private
    { Private declarations }
    FEmptyWin : Boolean;
  protected
    { Protected declarations }
  public
    { Public declarations }
    Form       : TForm;
    StringGrid : TStringGrid;

    Constructor Create(AOwner:TComponent); OverRide;

    Procedure   Action;  OverRide;       {Go Action}
  published
    { Published declarations }
    property EmptyWin : Boolean read FEmptyWin write FEmptyWin;
  end;

  TMADWinParamManager = class(TWinParamManager)
  private
     PF          : TParamForm;
  public
     Constructor Create(AOwner:TComponent); OverRide;
  end;

  TListParamManager = class(TAbstractParamManager)
  private
    { Private declarations }
    FList : TStringList;

    Procedure SetList(aList:TStringList);
  protected
    { Protected declarations }
  public
    { Public declarations }
    Constructor Create(AOwner:TComponent); OverRide;
    Destructor  Destroy; OverRide;

    Procedure   Action;  OverRide;       {Go Action}
  published
    { Published declarations }
    Property List : TStringList read FList write SetList;
  end;


procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Mad Soft', [TWinParamManager]);
  RegisterComponents('Mad Soft', [TMADWinParamManager]);
  RegisterComponents('Mad Soft', [TListParamManager]);
end;

{===============================}

Constructor TAbstractParamManager.Create(AOwner:TComponent);
   Begin
      Inherited Create(AOwner);

      Accepted:=False;

      M:=TKeyArr.Create;
   End;

Destructor  TAbstractParamManager.Destroy;
   Begin
      M.Destroy;

      Inherited Destroy;
   End;

Procedure   TAbstractParamManager.Clear;
   Begin
      M.Clear;

      Accepted:=False;
   End;

Procedure   TAbstractParamManager.Get(Q:TQuery);
   Var N : String;
       I : Word;
   Begin
      With Q Do
         If ParamCount > 0 Then
            For I:=0 To ParamCount - 1 Do
               Begin
                  N:=Params[I].Name;
                  If N[1] = ParamManagerSymbol Then
                     Begin
                        System.Delete(N,1,1);
                        M.Add(N);
                     End;
               End;
      Accepted:=False;
   End;

Procedure   TAbstractParamManager.Action;
   Begin
      Accepted:=True;
   End;

Procedure   TAbstractParamManager.Accept(Q:TQuery);
   Var N  : String;
       I  : Word;
       PT : TFieldType;
   Begin
      If Not Accepted Then
         Exit;

      With Q Do
         If ParamCount > 0 Then
            For I:=0 To ParamCount - 1 Do
               Begin
                  N:=Params[I].Name;
                  If N[1] = ParamManagerSymbol Then
                     Begin
                        Case UpCase(N[2]) Of
                           'S':PT:=ftString;
                           'I':PT:=ftInteger;
                           'W':PT:=ftWord;
                           'F':PT:=ftFloat;
                           'D':PT:=ftDate;
                           'T':PT:=ftTime;
                           'B':PT:=ftBoolean;
                        Else{CASE}
                           PT:=ftUnknown;
                        End;{CASE}

                        Params[I].DataType:=PT;

                        System.Delete(N,1,1);
                        If M.Exist(N) <> 0 Then
                           Params[I].Text:=M.GetValue(N)
                        Else
                           Params[I].Clear;
                     End;
               End;
   End;

{============================================================}

Constructor TWinParamManager.Create(AOwner:TComponent);
   Begin
      Inherited Create(AOwner);

      FEmptyWin:=False;

      Form:=Nil;
      StringGrid:=Nil;
   End;

Procedure   TWinParamManager.Action;
   Var I  : Word;
       N  : String;
   Begin
      If (StringGrid = Nil) Or
         (Form = Nil      ) Then
         Exit;

      With StringGrid Do
         Begin
            { ColCount:=3; //Do not change or unremark!!! }
            FixedCols:=2;

            If M.Br > 0 Then
               RowCount:=1 + M.Br
            Else
               RowCount:=2;

            FixedRows:=1;

            Cells[0,0]:='Param';
            Cells[1,0]:='T';
            Cells[2,0]:='Value';

            If M.Br > 0 Then
               For I:=1 To M.Br Do
                  Begin
                     N:=M.M[I].Key;

                     Cells[1,I]:=N[1] + '';

                     Delete(N,1,1);

                     Cells[0,I]:=N;
                     Cells[2,I]:=M.M[I].Value;
                  End
            Else
               Begin
                  Cells[0,1]:='[No Params]';
                  Cells[1,1]:='';
                  Cells[2,1]:='';
               End;
         End;

      If (M.Br > 0) Or
         (EmptyWin) Then
         Form.ShowModal;

      Accepted:= (Form.ModalResult = MrOK) Or
                 (M.Br = 0);

      If Accepted Then
         With StringGrid Do
            For I:=1 To M.Br Do
               M.SetValue(Cells[1,I] + Cells[0,I],Cells[2,I]);
   End;

{==========================================================}

Constructor TMADWinParamManager.Create(AOwner:TComponent);
   Begin
      Inherited Create(AOwner);

      PF:=TParamForm.Create(Self);

      Form:=PF;
      StringGrid:=PF.StringGrid1;
   End;

{==========================================================}

Constructor TListParamManager.Create(AOwner:TComponent);
   Begin
      Inherited Create(AOwner);

      FList:=TStringList.Create;
   End;

Destructor TListParamManager.Destroy;
   Begin
      FList.Destroy;

      Inherited Destroy;
   End;

Procedure TListParamManager.SetList(aList:TStringList);
   Begin
      FList.Assign(aList);
   End;

Procedure   TListParamManager.Action;
   Var I : Byte;
   Begin
      For I:=1 To M.Br Do
         M.M[I].Value:=FList.Values[M.M[I].Key];
      Accepted:=True;
   End;

end.

